#include "config.h"

struct cfg_tab {
		int unit, addr, irq;
	};

struct cfg_tab snd_cfg_tab[] = {

#ifdef SND_0_VECT
		{SND_0, SND_0_SIOA, SND_0_VECT},
#endif
#ifdef SND_1_VECT
		{SND_1, SND_1_SIOA, SND_1_VECT},
#endif
#ifdef SND_2_VECT
		{SND_2, SND_2_SIOA, SND_2_VECT},
#endif
#ifdef SND_3_VECT
		{SND_3, SND_3_SIOA, SND_3_VECT},
#endif
#ifdef SND_4_VECT
		{SND_4, SND_4_SIOA, SND_4_VECT},
#endif
#ifdef SND_5_VECT
		{SND_5, SND_5_SIOA, SND_5_VECT},
#endif
#ifdef SND_6_VECT
		{SND_6, SND_6_SIOA, SND_6_VECT},
#endif
#ifdef SND_7_VECT
		{SND_7, SND_7_SIOA, SND_7_VECT},
#endif
		{-1, 0, 0}
	};
